@@ -1,11 +1,24 @@
-import { attributeRules } from "./attributes";
-import { compilePseudoSelector } from "./pseudo-selectors";
+import { attributeRules } from "./attributes.js";
+import { compilePseudoSelector } from "./pseudo-selectors/index.js";
 import type {
+    Adapter,
     CompiledQuery,
     InternalOptions,
     InternalSelector,
     CompileToken,
-} from "./types";
+} from "./types.js";
+import { SelectorType } from "css-what";
+
+function getElementParent<Node, ElementNode extends Node>(
+    node: ElementNode,
+    adapter: Adapter<Node, ElementNode>
+): ElementNode | null {
+    const parent = adapter.getParent(node);
+    if (parent && adapter.isTag(parent)) {
+        return parent;
+    }
+    return null;
+}
 
 /*
  * All available rules
@@ -21,13 +34,27 @@ export function compileGeneralSelector<Node, ElementNode extends Node>(
     const { adapter, equals } = options;
 
     switch (selector.type) {
-        case "pseudo-element":
+        case SelectorType.PseudoElement: {
             throw new Error("Pseudo-elements are not supported by css-select");
+        }
+        case SelectorType.ColumnCombinator: {
+            throw new Error(
+                "Column combinators are not yet supported by css-select"
+            );
+        }
+        case SelectorType.Attribute: {
+            if (selector.namespace != null) {
+                throw new Error(
+                    "Namespaced attributes are not yet supported by css-select"
+                );
+            }
 
-        case "attribute":
+            if (!options.xmlMode || options.lowerCaseAttributeNames) {
+                selector.name = selector.name.toLowerCase();
+            }
             return attributeRules[selector.action](next, selector, options);
-
-        case "pseudo":
+        }
+        case SelectorType.Pseudo: {
             return compilePseudoSelector(
                 next,
                 selector,
@@ -35,15 +62,28 @@ export function compileGeneralSelector<Node, ElementNode extends Node>(
                 context,
                 compileToken
             );
-
+        }
         // Tags
-        case "tag":
+        case SelectorType.Tag: {
+            if (selector.namespace != null) {
+                throw new Error(
+                    "Namespaced tag names are not yet supported by css-select"
+                );
+            }
+
+            let { name } = selector;
+
+            if (!options.xmlMode || options.lowerCaseTags) {
+                name = name.toLowerCase();
+            }
+
             return function tag(elem: ElementNode): boolean {
-                return adapter.getName(elem) === selector.name && next(elem);
+                return adapter.getName(elem) === name && next(elem);
             };
+        }
 
         // Traversal
-        case "descendant":
+        case SelectorType.Descendant: {
             if (
                 options.cacheResults === false ||
                 typeof WeakSet === "undefined"
@@ -51,8 +91,8 @@ export function compileGeneralSelector<Node, ElementNode extends Node>(
                 return function descendant(elem: ElementNode): boolean {
                     let current: ElementNode | null = elem;
 
-                    while ((current = adapter.getParent(current))) {
-                        if (adapter.isTag(current) && next(current)) {
+                    while ((current = getElementParent(current, adapter))) {
+                        if (next(current)) {
                             return true;
                         }
                     }
@@ -62,12 +102,11 @@ export function compileGeneralSelector<Node, ElementNode extends Node>(
             }
 
             // @ts-expect-error `ElementNode` is not extending object
-            // eslint-disable-next-line no-case-declarations
             const isFalseCache = new WeakSet<ElementNode>();
             return function cachedDescendant(elem: ElementNode): boolean {
                 let current: ElementNode | null = elem;
 
-                while ((current = adapter.getParent(current))) {
+                while ((current = getElementParent(current, adapter))) {
                     if (!isFalseCache.has(current)) {
                         if (adapter.isTag(current) && next(current)) {
                             return true;
@@ -78,32 +117,33 @@ export function compileGeneralSelector<Node, ElementNode extends Node>(
 
                 return false;
             };
-        case "_flexibleDescendant":
+        }
+        case "_flexibleDescendant": {
             // Include element itself, only used while querying an array
             return function flexibleDescendant(elem: ElementNode): boolean {
                 let current: ElementNode | null = elem;
 
                 do {
-                    if (adapter.isTag(current) && next(current)) return true;
-                } while ((current = adapter.getParent(current)));
+                    if (next(current)) return true;
+                } while ((current = getElementParent(current, adapter)));
 
                 return false;
             };
-
-        case "parent":
+        }
+        case SelectorType.Parent: {
             return function parent(elem: ElementNode): boolean {
                 return adapter
                     .getChildren(elem)
                     .some((elem) => adapter.isTag(elem) && next(elem));
             };
-
-        case "child":
+        }
+        case SelectorType.Child: {
             return function child(elem: ElementNode): boolean {
                 const parent = adapter.getParent(elem);
                 return parent != null && adapter.isTag(parent) && next(parent);
             };
-
-        case "sibling":
+        }
+        case SelectorType.Sibling: {
             return function sibling(elem: ElementNode): boolean {
                 const siblings = adapter.getSiblings(elem);
 
@@ -117,8 +157,15 @@ export function compileGeneralSelector<Node, ElementNode extends Node>(
 
                 return false;
             };
+        }
+        case SelectorType.Adjacent: {
+            if (adapter.prevElementSibling) {
+                return function adjacent(elem: ElementNode): boolean {
+                    const previous = adapter.prevElementSibling!(elem);
+                    return previous != null && next(previous);
+                };
+            }
 
-        case "adjacent":
             return function adjacent(elem: ElementNode): boolean {
                 const siblings = adapter.getSiblings(elem);
                 let lastElement;
@@ -133,8 +180,15 @@ export function compileGeneralSelector<Node, ElementNode extends Node>(
 
                 return !!lastElement && next(lastElement);
             };
+        }
+        case SelectorType.Universal: {
+            if (selector.namespace != null && selector.namespace !== "*") {
+                throw new Error(
+                    "Namespaced universal selectors are not yet supported by css-select"
+                );
+            }
 
-        case "universal":
             return next;
+        }
     }
 }
