@@ -1,8 +1,12 @@
-import { CompileToken } from "./../types";
 import type { Selector } from "css-what";
-import { trueFunc, falseFunc } from "boolbase";
-import type { CompiledQuery, InternalOptions, Adapter } from "../types";
-import { isTraversal } from "../procedure";
+import boolbase from "boolbase";
+import type {
+    CompiledQuery,
+    InternalOptions,
+    CompileToken,
+    Adapter,
+} from "../types.js";
+import { isTraversal } from "../sort.js";
 
 /** Used as a placeholder for :has. Will be replaced with the actual element. */
 export const PLACEHOLDER_ELEMENT = {};
@@ -11,7 +15,7 @@ export function ensureIsTag<Node, ElementNode extends Node>(
     next: CompiledQuery<ElementNode>,
     adapter: Adapter<Node, ElementNode>
 ): CompiledQuery<Node> {
-    if (next === falseFunc) return falseFunc;
+    if (next === boolbase.falseFunc) return boolbase.falseFunc;
     return (elem: Node) => adapter.isTag(elem) && next(elem);
 }
 
@@ -34,15 +38,30 @@ export function getNextSiblings<Node, ElementNode extends Node>(
     return siblings.slice(elemIndex + 1).filter(adapter.isTag);
 }
 
-const is: Subselect = (next, token, options, context, compileToken) => {
-    const opts = {
+function copyOptions<Node, ElementNode extends Node>(
+    options: InternalOptions<Node, ElementNode>
+): InternalOptions<Node, ElementNode> {
+    // Not copied: context, rootFunc
+    return {
         xmlMode: !!options.xmlMode,
+        lowerCaseAttributeNames: !!options.lowerCaseAttributeNames,
+        lowerCaseTags: !!options.lowerCaseTags,
+        quirksMode: !!options.quirksMode,
+        cacheResults: !!options.cacheResults,
+        pseudos: options.pseudos,
         adapter: options.adapter,
         equals: options.equals,
     };
+}
 
-    const func = compileToken(token, opts, context);
-    return (elem) => func(elem) && next(elem);
+const is: Subselect = (next, token, options, context, compileToken) => {
+    const func = compileToken(token, copyOptions(options), context);
+
+    return func === boolbase.trueFunc
+        ? next
+        : func === boolbase.falseFunc
+        ? boolbase.falseFunc
+        : (elem) => func(elem) && next(elem);
 };
 
 /*
@@ -58,20 +77,13 @@ export const subselects: Record<string, Subselect> = {
     matches: is,
     where: is,
     not(next, token, options, context, compileToken) {
-        const opts = {
-            xmlMode: !!options.xmlMode,
-            adapter: options.adapter,
-            equals: options.equals,
-        };
+        const func = compileToken(token, copyOptions(options), context);
 
-        const func = compileToken(token, opts, context);
-
-        if (func === falseFunc) return next;
-        if (func === trueFunc) return falseFunc;
-
-        return function not(elem) {
-            return !func(elem) && next(elem);
-        };
+        return func === boolbase.falseFunc
+            ? next
+            : func === boolbase.trueFunc
+            ? boolbase.falseFunc
+            : (elem) => !func(elem) && next(elem);
     },
     has<Node, ElementNode extends Node>(
         next: CompiledQuery<ElementNode>,
@@ -81,45 +93,39 @@ export const subselects: Record<string, Subselect> = {
         compileToken: CompileToken<Node, ElementNode>
     ): CompiledQuery<ElementNode> {
         const { adapter } = options;
-        const opts = {
-            xmlMode: !!options.xmlMode,
-            adapter,
-            equals: options.equals,
-        };
 
-        // @ts-expect-error Uses an array as a pointer to the current element (side effects)
-        const context: ElementNode[] | undefined = subselect.some((s) =>
-            s.some(isTraversal)
-        )
-            ? [PLACEHOLDER_ELEMENT]
+        const opts = copyOptions(options);
+        opts.relativeSelector = true;
+
+        const context = subselect.some((s) => s.some(isTraversal))
+            ? // Used as a placeholder. Will be replaced with the actual element.
+              ([PLACEHOLDER_ELEMENT] as unknown as ElementNode[])
             : undefined;
 
         const compiled = compileToken(subselect, opts, context);
 
-        if (compiled === falseFunc) return falseFunc;
-        if (compiled === trueFunc) {
-            return (elem) =>
-                adapter.getChildren(elem).some(adapter.isTag) && next(elem);
-        }
+        if (compiled === boolbase.falseFunc) return boolbase.falseFunc;
 
         const hasElement = ensureIsTag(compiled, adapter);
 
-        const { shouldTestNextSiblings = false } = compiled;
+        // If `compiled` is `trueFunc`, we can skip this.
+        if (context && compiled !== boolbase.trueFunc) {
+            /*
+             * `shouldTestNextSiblings` will only be true if the query starts with
+             * a traversal (sibling or adjacent). That means we will always have a context.
+             */
+            const { shouldTestNextSiblings = false } = compiled;
 
-        /*
-         * `shouldTestNextSiblings` will only be true if the query starts with
-         * a traversal (sibling or adjacent). That means we will always have a context.
-         */
-        if (context) {
             return (elem) => {
+                if (!next(elem)) return false;
+
                 context[0] = elem;
                 const childs = adapter.getChildren(elem);
                 const nextElements = shouldTestNextSiblings
                     ? [...childs, ...getNextSiblings(elem, adapter)]
                     : childs;
-                return (
-                    next(elem) && adapter.existsOne(hasElement, nextElements)
-                );
+
+                return adapter.existsOne(hasElement, nextElements);
             };
         }
 
