@@ -32,7 +32,7 @@ export interface Adapter<Node, ElementNode extends Node> {
     /**
      * Get the parent of the node
      */
-    getParent: (node: ElementNode) => ElementNode | null;
+    getParent: (node: ElementNode) => Node | null;
 
     /**
      * Get the siblings of the node. Note that unlike jQuery's `siblings` method,
@@ -40,6 +40,11 @@ export interface Adapter<Node, ElementNode extends Node> {
      */
     getSiblings: (node: Node) => Node[];
 
+    /**
+     * Returns the previous element sibling of a node.
+     */
+    prevElementSibling?: (node: Node) => ElementNode | null;
+
     /**
      * Get the text content of the node, and its children if it has any.
      */
@@ -102,6 +107,40 @@ export interface Options<Node, ElementNode extends Node> {
      * @default false
      */
     xmlMode?: boolean;
+    /**
+     * Lower-case attribute names.
+     *
+     * @default !xmlMode
+     */
+    lowerCaseAttributeNames?: boolean;
+    /**
+     * Lower-case tag names.
+     *
+     * @default !xmlMode
+     */
+    lowerCaseTags?: boolean;
+    /**
+     * Is the document in quirks mode?
+     *
+     * This will lead to .className and #id being case-insensitive.
+     *
+     * @default false
+     */
+    quirksMode?: boolean;
+    /**
+     * Pseudo-classes that override the default ones.
+     *
+     * Maps from names to either strings of functions.
+     * - A string value is a selector that the element must match to be selected.
+     * - A function is called with the element as its first argument, and optional
+     *  parameters second. If it returns true, the element is selected.
+     */
+    pseudos?:
+        | Record<
+              string,
+              string | ((elem: ElementNode, value?: string | null) => boolean)
+          >
+        | undefined;
     /**
      * The last function in the stack, will be called with the last element
      * that's looked at.
@@ -114,9 +153,23 @@ export interface Options<Node, ElementNode extends Node> {
     adapter?: Adapter<Node, ElementNode>;
     /**
      * The context of the current query. Used to limit the scope of searches.
-     * Can be matched directly using the `:scope` pseudo-selector.
+     * Can be matched directly using the `:scope` pseudo-class.
      */
     context?: Node | Node[];
+    /**
+     * Indicates whether to consider the selector as a relative selector.
+     *
+     * Relative selectors that don't include a `:scope` pseudo-class behave
+     * as if they have a `:scope ` prefix (a `:scope` pseudo-class, followed by
+     * a descendant selector).
+     *
+     * If relative selectors are disabled, selectors starting with a traversal
+     * will lead to an error.
+     *
+     * @default true
+     * @see {@link https://www.w3.org/TR/selectors-4/#relative}
+     */
+    relativeSelector?: boolean;
     /**
      * Allow css-select to cache results for some selectors, sometimes greatly
      * improving querying performance. Disable this if your document can
