@@ -1,6 +1,6 @@
-import * as helper from "./helper";
+import * as helper from "./helper.js";
 import CSSselect from "../../src";
-import type { Element, Node } from "domhandler";
+import type { Element, Node, Document } from "domhandler";
 let document = loadDoc();
 
 export function loadDoc(): helper.SimpleDocument {
@@ -30,16 +30,16 @@ export function t(
     context: Node[] | Node | null = document
 ): void {
     const actual = CSSselect(selector, context) as Element[];
-    const actualIds = actual.map((e) => e.attribs.id);
+    const actualIds = actual.map((e) => e.attribs["id"]);
 
     // Should not contain falsy values
     expect(actualIds).toStrictEqual(expectedIds);
 }
 
-const xmlDoc = helper.getDOMFromPath("fries.xml", {
+const xmlDoc = helper.getDocumentFromPath("fries.xml", {
     xmlMode: true,
 });
 
-export function createWithFriesXML(): Node[] {
+export function createWithFriesXML(): Document {
     return xmlDoc;
 }
